﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.MapSuiteGeocoder;

namespace ThinkGeoGIS_PoC.Forms
{
    public class Address
    {
        #region Private Instance Fields

        private PointShape _GISLocation = null;
        private string _AddressText = null;
        private MatchItem matchItem;
        private IEnumerable<string> keyList;

        #endregion

        #region Constructors

        public Address()
        {

        }

        public Address(MatchItem matchItem, IEnumerable<string> keyList)
        {
            this.matchItem = matchItem;
            this.keyList = keyList;
            this.AddressText = this.ToString();
            //this.GISLocation = (PointShape)PointShape.CreateShapeFromWellKnownData(matchItem.CentroidWkt);
        }

        public Address(string address)
        {
            this.AddressText = address;
        }

        public Address(PointShape GISPoint)
        {
            this.GISLocation = GISPoint;
        }

        public Address(string address, PointShape GISPoint)
        {
            this.AddressText = address;
            this.GISLocation = GISPoint;
        }

        public Address(string address, MatchItem matchItem, IEnumerable<string> keyList)
        {
            this.MatchItem = matchItem;
            this.keyList = keyList;
            this.AddressText = address;
            this.GISLocation = (PointShape)PointShape.CreateShapeFromWellKnownData(matchItem.CentroidWkt);
        }

        #endregion

        #region Public Instance Properties

        public MatchItem MatchItem
        {
            get { return matchItem; }
            set { matchItem = value; }
        }

        public string AddressText 
        {
            get
            {
                if (String.IsNullOrEmpty(_AddressText))
                {
                    if (String.IsNullOrEmpty(this.ToString()))
                        _AddressText = String.Empty;
                    else
                        _AddressText = this.ToString();
                }
                return _AddressText;
            }
            set
            {
                string temp = value.Trim();
                _AddressText = temp.ToUpper();
            } 
        }

        public PointShape GISLocation
        {
            get
            {
                if (_GISLocation == null)
                    _GISLocation = new PointShape();
                return _GISLocation;
            }
            set
            {
                _GISLocation = value;
            }
        }

        #endregion

        #region Public Instance Methods



        #endregion

        #region Private Instance Methods



        #endregion

        #region Public Overrides

        public override string ToString()
        {
            if (MatchItem == null)
                return "";
            StringBuilder keyValuesBuilder = new StringBuilder();
            foreach (string item in keyList)
            {
                if (!MatchItem.MatchPairs.ContainsKey(item))
                    break;

                string temp = MatchItem.MatchPairs[item].ToUpper();
                if (!String.IsNullOrEmpty(temp) && temp != "NONE")
                    keyValuesBuilder.Append(string.Format("{0} ", temp));

                // Original from sample
                //keyValuesBuilder.Append(string.Format("{0} ", MatchItem.MatchPairs[item]));
            }

            return keyValuesBuilder.ToString();
        }

        #endregion

    }
}
