﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.MapSuiteGeocoder;
using System.IO;
using System.Collections.ObjectModel;

namespace ThinkGeoGIS_PoC.Forms
{
    public partial class AddressSelectionDialog : Form
    {

        #region Private Instance Fields

        private Collection<MatchResult> _Matches = null;
        private Collection<MatchItem> _MatchItems = null;
        private MatchItem _SelectedLocation = null;

        #endregion

        #region Constructors

        public AddressSelectionDialog()
        {
            InitializeComponent();
            this.Load += new EventHandler(AddressSelectionDialog_Load);
            this.FormClosing += new FormClosingEventHandler(AddressSelectionDialog_FormClosing);
            this.btnOK.Click += new EventHandler(btnOK_Click);
            this.lstMatches.DoubleClick += new EventHandler(lstMatches_DoubleClick);
            this.lstMatches.Click += new EventHandler(lstMatches_Click);
        }

        
        #endregion

        #region Events

        protected void AddressSelectionDialog_FormClosing(object sender, FormClosingEventArgs e)
        {
            //if (lstMatches.SelectedItem == null)
            //{
            //    MessageBox.Show("You must pick at least one address");
            //    return;
            //}
        }

        protected void lstMatches_Click(object sender, EventArgs e)
        {
            SetLocation();
        }

        protected void lstMatches_DoubleClick(object sender, EventArgs e)
        {
            SetLocation();
            btnOK_Click(this, new EventArgs());
        }

        protected void AddressSelectionDialog_Load(object sender, EventArgs e)
        {
            if (MatchItems != null && MatchItems.Count > 0)
                PopulateAddressResultListByMatchItems();
            else if (Matches.Count > 0)
                PopulateAddressResultListByMatches();
            else
                this.DialogResult = DialogResult.Cancel;
        }

        protected void btnOK_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.OK;
        }

        #endregion

        #region Public Instance Properties

        public Collection<MatchResult> Matches
        {
            get
            {
                if (_Matches == null)
                    _Matches = new Collection<MatchResult>();
                return _Matches;
            }
            set
            {
                _Matches = value;
            }
        }

        public Collection<MatchItem> MatchItems
        {
            get
            {
                if (_MatchItems == null)
                    _MatchItems = new Collection<MatchItem>();
                return _MatchItems;
            }
            set
            {
                _MatchItems = value;
            }
        }


        public MatchItem SelectedLocation { get; set; }
        //{
        //    get
        //    {
        //        if (_SelectedLocation == null)
        //            _SelectedLocation = new MatchItem();
        //        return _SelectedLocation;
        //    }
        //    set
        //    {
        //        _SelectedLocation = value;
        //    }
        //}

        #endregion

        #region Public Instance Methods



        #endregion

        #region Private Instance Methods

        private void PopulateAddressResultListByMatches()
        {
            foreach (MatchResult matchResult in Matches)
            {
                if (matchResult.MatchItems.Count == 0)
                    break;

                // Load the matching items into the grid
                foreach (MatchItem matchItem in matchResult.MatchItems)
                {
                    lstMatches.Items.Add(new ListItem(matchItem, new string[] { "HouseNumber", "PrefixDirectional", "Street", "StreetType", "SuffixDirectional", "City" }));
                }
            }
        }

        private void PopulateAddressResultListByMatchItems()
        {
            // Load the matching items into the grid
            foreach (MatchItem matchItem in MatchItems)
            {
                lstMatches.Items.Add(new ListItem(matchItem, new string[] { "HouseNumber", "PrefixDirectional", "Street", "StreetType", "SuffixDirectional", "City" }));
            }
        }

        private void SetLocation()
        {
            if (lstMatches.SelectedItem == null)
                return;

            ListItem item = lstMatches.SelectedItem as ListItem;
            SelectedLocation = item.MatchItem;
        }

        #endregion
        
    }
}
