﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.DesktopEdition;
using System.IO;

namespace ThinkGeoGIS_PoC
{
    public partial class HelloWorld : Form
    {
        #region Constructors

        public HelloWorld()
        {
            InitializeComponent();

            this.Load += new EventHandler(MainForm_Load);
        }

        #endregion

        #region Events

        protected void MainForm_Load(object sender, EventArgs e)
        {
            string mapbase = Path.Combine(Environment.CurrentDirectory, "MapData");
            // Set the Map Unit. The reason for setting it to DecimalDegrees is that is what the shapefile’s unit of measure is inherently in.
            wfmapMap.MapUnit = GeographyUnit.DecimalDegree;

            // We create a new Layer and pass the path to a Shapefile into its constructor. 
            string mapfile = Path.Combine(mapbase, "Countries02.shp");
            ShapeFileFeatureLayer worldLayer = new ShapeFileFeatureLayer(mapfile);

            // Set the worldLayer with a preset Style, as AreaStyles.Country1 has YellowGreen background and black border, our worldLayer will have the same render style.  
            worldLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.Country1;

            // This setting will apply from ZoonLevel01 to ZoomLevel20, that means we can see the world the same style with ZoomLevel01 all the time no matter how far we zoom out/in. 
            worldLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            // Capitals
            ShapeFileFeatureLayer capitalLayer = new ShapeFileFeatureLayer(Path.Combine(mapbase, "WorldCapitals.shp"));
            capitalLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = PointStyles.Capital1;
            capitalLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            // Create a new Layer Overlay to hold the layer we just created
            LayerOverlay layerOverlay = new LayerOverlay();

            // Add the shapefile layer to the layer overlay
            layerOverlay.Layers.Add(worldLayer);
            layerOverlay.Layers.Add(capitalLayer);

            // We need to add the layer overlay to Map.
            wfmapMap.Overlays.Add(layerOverlay);

            // Set a proper extent for the Map.  
            wfmapMap.CurrentExtent = new RectangleShape(0, 78, 30, 26);

            // We now need to call the Refresh() method of the Map control so that the Map can redraw based on the data that has been provided.
            wfmapMap.Refresh();

        }

        #endregion
        
    }
}
