﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.DesktopEdition;
using ThinkGeo.MapSuite.MapSuiteGeocoder;
using System.IO;
using System.Collections.ObjectModel;
using System.Text.RegularExpressions;
using System.Threading;

namespace ThinkGeoGIS_PoC.Forms
{
    public partial class VerifyMap : Form
    {
        

        #region Private Instance Fields

        private string mapbase = Path.Combine(Environment.CurrentDirectory, "MapData");
        private string INmapbase = @"C:\Indiana Map Data\IN";
        private string GeocoderPath = @"C:\ThinkGeo\GeocodeData\Map Suite 3.0 Geocoder Index";
        private string CountyPath;
        private UsaGeocoder _VerifyGeocoder = null;
        private List<Address> _Locations = null;
        private AutoCompleteStringCollection _AutoCompleteLocations = null;
        private List<string> _CountyZips = null;
        private int TestCounter = 0;

        private Dictionary<string, int> _Counties = null;

        private const string CountyExt = "ctycu.shp";
        private const string CityExt = "placecu.shp";
        private const string RoadsExt = "lkA.shp";
        private const string RailExt = "lkB.shp";
        private const string WaterExt = "wat.shp";

        private const string HighwayIdxExt = "Highway.idx";
        private const string SecondaryIdxExt = "Secondary.idx";
        private const string LocalIdxExt = "LocalRoad.idx";

        private string ShpFilenameCounty;
        private string ShpFilenameCity;
        private string ShpFilenameRoads;
        private string ShpFilenameRail;
        private string ShpFilenameWater;

        private string IdxFilenameHighway;
        private string IdxFilenameSecondary;
        private string IdxFilenameLocal;

        #endregion

        #region Constructors

        public VerifyMap()
        {
            InitializeComponent();
            this.Load += new EventHandler(VerifyMap_Load);
            //this.txtLocation.TextChanged += new EventHandler(txtLocation_TextChanged); // Commented out until issue with matching not being limited to Zips is resolved.
            this.txtLocation.KeyDown += new KeyEventHandler(txtLocation_KeyDown);
            this.mapMap.MapClick += new EventHandler<MapClickWinformsMapEventArgs>(mapMap_MapClick);
            this.mapMap.CurrentExtentChanging += new EventHandler<CurrentExtentChangingWinformsMapEventArgs>(mapMap_CurrentExtentChanging);
            this.btnConfirm.Click += new EventHandler(btnConfirm_Click);
            this.FormClosing += new FormClosingEventHandler(VerifyMap_FormClosing);
            this.btnZoomIn.Click+=new EventHandler(btnZoomIn_Click);
            this.btnZoomOut.Click+=new EventHandler(btnZoomOut_Click);
            this.btnMaxExtent.Click += new EventHandler(btnMaxExtent_Click);
            this.cmbCounty.SelectedIndexChanged += new EventHandler(cmbCounty_SelectedIndexChanged);
        }

        #endregion

        #region Events

        protected void btnMaxExtent_Click(object sender, EventArgs e)
        {
            SetMaxView();
        }

        protected void mapMap_CurrentExtentChanging(object sender, CurrentExtentChangingWinformsMapEventArgs e)
        {
            if (e.NewExtent.GetPerimeter(GeographyUnit.Meter, DistanceUnit.Meter) > GetMaxExtent().GetPerimeter(GeographyUnit.Meter, DistanceUnit.Meter))
            {
                SetMaxView();
                e.Cancel = true;
            }
        }

        protected void cmbCounty_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (cmbCounty.SelectedValue == null)
                return;

            SetMapFilenames((int)cmbCounty.SelectedValue);
            SetupMap();
        }

        protected void VerifyMap_FormClosing(object sender, FormClosingEventArgs e)
        {
            UserEnteredLocation = txtLocation.Text;
        }

        protected void btnConfirm_Click(object sender, EventArgs e)
        {
            //// Test after other layers of same file are open - This DOES NOT work in method other than setup map.
            //ShapeFileFeatureLayer AllRoads = new ShapeFileFeatureLayer(Path.Combine(INmapbase, @"Elkhart\IN039lkA.shp"));
            //AllRoads.Open();
            //DataTable dtTest = AllRoads.QueryTools.ExecuteQuery("Select FENAME From IN039lkA Where FENAME Like('%County%');");
            //Console.WriteLine(dtTest.Rows.Count);
            ////foreach (DataRow row in dtTest.Rows)
            ////{
            ////    Console.WriteLine(row["FENAME"]); // Just output to console for testing
            ////}
            //AllRoads.Close();
            //this.DialogResult = DialogResult.OK;
        }

        /// <summary>
        /// Places an icon on the map for a clicked location. Also will reverse geocode the closest address.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void mapMap_MapClick(object sender, MapClickWinformsMapEventArgs e)
        {            
            pbVerified.Visible = true;
            Verified = true;
            LayerOverlay markerOverlay = mapMap.Overlays["MarkerOverlay"] as LayerOverlay;
            MarkerLayer markerLayer = markerOverlay.Layers["MarkerLayer"] as MarkerLayer;
            markerLayer.MarkerLocation = new PointShape(e.WorldX, e.WorldY);

            // Reverse Geocode Location
            UsaGeocoder usaGeocoder = new UsaGeocoder(GeocoderPath);

            MatchResult matchResult = new MatchResult();
            try
            {
                usaGeocoder.Open();
                matchResult = usaGeocoder.Match(String.Format("{0} {1}", e.WorldX.ToString(), e.WorldY.ToString()));
            }
            catch (Exception ex)
            {
                MessageBox.Show(String.Format("RevGeocode error:\n{0}", ex.ToString()));
            }
            finally
            {
                usaGeocoder.Close();
            }

            if (matchResult.MatchItems.Count > 0)
            {
                MatchItem matchItem = matchResult.MatchItems[0];
                Address address = new Address(matchItem, new string[] { "HouseNumber", "PrefixDirectional", "Street", "StreetType", "SuffixDirectional", "City" });
                if (
                    MessageBox.Show(
                        String.Format("Closest address is {0}. Do you want to change your location text?", address.AddressText),
                        "Address Found", MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button1) == DialogResult.Yes
                    )
                {
                    txtLocation.Text = address.AddressText;
                }
                else
                    return;

                //using (AddressSelectionDialog asd = new AddressSelectionDialog())
                //{
                //    asd.MatchItems = matchResult.MatchItems;
                //    if (asd.ShowDialog() != DialogResult.Cancel)
                //    {
                //        Address address = new Address(asd.SelectedLocation, new string[] { "HouseNumber", "PrefixDirectional", "Street", "StreetType", "SuffixDirectional", "City" });
                //        txtLocation.Text = address.AddressText;
                //    }
                //}
            }

            mapMap.Refresh(mapMap.Overlays["MarkerOverlay"]);
        }

        protected void btnZoomOut_Click(object sender, EventArgs e)
        {
            mapMap.ZoomOut(40);
            mapMap.Refresh();
        }

        protected void btnZoomIn_Click(object sender, EventArgs e)
        {
            mapMap.ZoomIn(50);
            mapMap.Refresh();
        }

        protected void txtLocation_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Return || e.KeyCode == Keys.Enter)
            {
                // Geocode address
                GeocodeLocation();
            }
        }

        protected void txtLocation_TextChanged(object sender, EventArgs e)
        {
            List<MatchResult> MatchResults = new List<MatchResult>();
            try
            {
                VerifyGeocoder.Open();
                //// This code was an attempt to limit the geocoding by Zip code, however it still returned results from all around the US.
                //foreach (string zip in CountyZips)
                //{
                //    MatchResults.Add(VerifyGeocoder.Match(txtLocation.Text, zip));
                //}

                Collection<MatchResult> matches = new Collection<MatchResult>();
                foreach (string zip in CountyZips)
                {
                    MatchResult temp = VerifyGeocoder.Match(txtLocation.Text, zip);
                    if (temp != null && temp.MatchItems.Count > 0)
                        matches.Add(temp);
                }

                foreach (MatchResult match in matches)
                {
                        if (match.MatchItems.Count > 0)
                        {
                            foreach (MatchItem item in match.MatchItems)
                            {
                                //Console.WriteLine(item);
                                if (CountyZips.Where(c => c == item.MatchPairs["Zip"]).Count() > 0 && item.MatchPairs["County"] == CountyName.ToUpper())
                                {
                                    MatchResults.Add(match);
                                }
                            }
                        }
                }

                //MatchResults.Add(VerifyGeocoder.Match(txtLocation.Text));
                
            }
            catch (Exception ex)
            {
                MessageBox.Show(String.Format("Geocode error:\n{0}", ex.ToString()));
            }
            finally
            {
                VerifyGeocoder.Close();
            }

            FillAutoCompleteList(MatchResults);
        }

        protected void VerifyMap_Load(object sender, EventArgs e)
        {
            FillCountyDictionary();
            BindCountyCombo();
            if (cmbCounty.SelectedValue == null)
                SetMapFilenames(20); // default to Elkhart for testing
            else
                SetMapFilenames((int)cmbCounty.SelectedValue);
            //SetupMap();
            pbVerified.Visible = false;
            txtLocation.Text = UserEnteredLocation; // this is in relation to another form used for testing but not needed with this form.
        }

        #endregion

        #region Public Instance Properties

        public string CountyName { get; set; }
        public string UserEnteredLocation { get; set; }
        public bool Verified { get; set; }

        #endregion

        #region Private Instance Properties

        private Dictionary<string, int> Counties
        {
            get
            {
                if (_Counties == null)
                    _Counties = new Dictionary<string, int>();
                return _Counties;
            }
            set
            {
                _Counties = value;
            }
        }

        private UsaGeocoder VerifyGeocoder
        {
            get
            {
                if (_VerifyGeocoder == null)
                    _VerifyGeocoder = new UsaGeocoder(GeocoderPath, MatchStatus.FuzzyMatch);
                return _VerifyGeocoder;
            }
        }

        private List<Address> Locations
        {
            get
            {
                if (_Locations == null)
                    _Locations = new List<Address>();
                return _Locations;
            }
            set
            {
                _Locations = value;
            }
        }

        private AutoCompleteStringCollection AutoCompleteLocations
        {
            get
            {
                if (_AutoCompleteLocations == null)
                    _AutoCompleteLocations = new AutoCompleteStringCollection();
                return _AutoCompleteLocations;
            }
            set
            {
                _AutoCompleteLocations = value;
            }
        }

        private List<string> CountyZips 
        {
            get
            {
                if (_CountyZips == null)
                    _CountyZips = new List<string>();
                return _CountyZips;
            }
            set
            {
                _CountyZips = value;
            }
        }

        #endregion

        #region Private Instance Methods

        private void FillCountyDictionary()
        {
            Counties.Add("Adams", 1);
            Counties.Add("Allen", 2);
            Counties.Add("Bartholomew", 3);
            Counties.Add("Elkhart", 20);
            Counties.Add("Hamilton", 29);
            Counties.Add("Lake", 45); 
            Counties.Add("Marion", 49);
            Counties.Add("St. Joseph", 71);
        }

        private void SetMapFilenames(int CountyNumber)
        {
            string GISCode = string.Empty;
            switch (CountyNumber)
            {
                case 1:
                    GISCode = "IN001"; // Adams
                    break;
                case 2:
                    GISCode = "IN003"; // Allen
                    break;
                case 3:
                    GISCode = "IN005"; // Bartholomew
                    break;
                case 20:
                    GISCode = "IN039"; // Elkhart
                    break;
                case 29:
                    GISCode = "IN057"; // Hamilton
                    break;
                case 45:
                    GISCode = "IN089"; // Lake
                    break;
                case 49:
                    GISCode = "IN097"; // Marion
                    break;
                case 71:
                    GISCode = "IN141"; // St. Joseph
                    break;
                default:
                    GISCode = "IN039"; // Default Elkhart for testing
                    break;
            }

            CountyName = Counties.Where(c => c.Value == CountyNumber).FirstOrDefault().Key;
            CountyPath = Path.Combine(INmapbase, Counties.Where(c => c.Value == CountyNumber).FirstOrDefault().Key);
            
            ShpFilenameCounty = Path.Combine(CountyPath, GISCode + CountyExt);
            ShpFilenameCity = Path.Combine(CountyPath, GISCode + CityExt);
            ShpFilenameRoads = Path.Combine(CountyPath, GISCode + RoadsExt);
            ShpFilenameRail = Path.Combine(CountyPath, GISCode + RailExt);
            ShpFilenameWater = Path.Combine(CountyPath, GISCode + WaterExt);

            IdxFilenameHighway = Path.Combine(CountyPath, GISCode + HighwayIdxExt);
            IdxFilenameSecondary = Path.Combine(CountyPath, GISCode + SecondaryIdxExt);
            IdxFilenameLocal = Path.Combine(CountyPath, GISCode + LocalIdxExt);
        }

        private void BindCountyCombo()
        {
            cmbCounty.DataSource = Counties.ToList<KeyValuePair<string, int>>();
        }

        /// <summary>
        /// One of our goals is to have the location textbox include an autocomplete suggestion list as the user enters their address.
        /// </summary>
        /// <param name="MatchResults"></param>
        private void FillAutoCompleteList(List<MatchResult> MatchResults)
        {
            if (MatchResults.Count < 1)
            {
                return;
            }

            foreach (MatchResult match in MatchResults)
            {
                if (match.MatchItems.Count > 0)
                {
                    foreach (MatchItem matchItem in match.MatchItems)
                    {
                        Address temp = new Address(matchItem, new string[] { "HouseNumber", "PrefixDirectional", "Street", "StreetType", "SuffixDirectional", "City" });
                        if (!String.IsNullOrEmpty(temp.AddressText))
                        {
                            if (!Locations.Contains(temp))
                                Locations.Add(temp);
                            if (!AutoCompleteLocations.Contains(temp.AddressText))
                            {
                                Console.WriteLine(temp.AddressText);
                                AutoCompleteLocations.Add(temp.AddressText);
                            }
                        }
                    }
                }
            }

            
            txtLocation.AutoCompleteCustomSource = AutoCompleteLocations;
        }

        private void SetupMap()
        {
            mapMap.Overlays.Clear();
            mapMap.MapUnit = GeographyUnit.DecimalDegree;
            mapMap.ZoomLevelSnapping = ZoomLevelSnappingMode.None;
            mapMap.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(GeoColor.GeographicColors.Sand);

            //// Map marking of location is not involved with current test/issue
            //InMemoryFeatureLayer pointLayer = new InMemoryFeatureLayer();
            //pointLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle.PointType = PointType.Bitmap;
            //pointLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle.Image = new GeoImage(Path.Combine(mapbase, "Semi Truck.png"));
            //pointLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            //ShapeFileFeatureLayer CountyLayer = new ShapeFileFeatureLayer(Path.Combine(INmapbase, @"Elkhart\IN039ctycu.shp"), ShapeFileReadWriteMode.ReadOnly);
            ShapeFileFeatureLayer CountyLayer = new ShapeFileFeatureLayer(ShpFilenameCounty, ShapeFileReadWriteMode.ReadOnly);
            CountyLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.County1;
            CountyLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = LineStyles.StateBorder1;
            CountyLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            //ShapeFileFeatureLayer CityLayer = new ShapeFileFeatureLayer(Path.Combine(INmapbase, @"Elkhart\IN039placecu.shp"), ShapeFileReadWriteMode.ReadOnly);
            ShapeFileFeatureLayer CityLayer = new ShapeFileFeatureLayer(ShpFilenameCity, ShapeFileReadWriteMode.ReadOnly);
            CityLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.Urban1;
            CityLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle = TextStyles.City1("NAME");
            CityLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = LineStyles.StateBorder1;
            CityLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level15;
            CityLayer.ZoomLevelSet.ZoomLevel15.DefaultAreaStyle = AreaStyles.Urban1;
            CityLayer.ZoomLevelSet.ZoomLevel15.DefaultLineStyle = LineStyles.StateBorder1;
            CityLayer.ZoomLevelSet.ZoomLevel15.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            
            //ShapeFileFeatureLayer HighwayLayer = new ShapeFileFeatureLayer(Path.Combine(INmapbase, @"Elkhart\IN039lkA.shp"), Path.Combine(INmapbase, @"Elkhart\IN039Highway.idx"), ShapeFileReadWriteMode.ReadOnly);
            ShapeFileFeatureLayer HighwayLayer = new ShapeFileFeatureLayer(ShpFilenameRoads, IdxFilenameHighway, ShapeFileReadWriteMode.ReadOnly);
            HighwayLayer.ZoomLevelSet.ZoomLevel12.DefaultLineStyle = LineStyles.Highway3;
            HighwayLayer.ZoomLevelSet.ZoomLevel12.DefaultTextStyle = TextStyles.Highway2("FENAME");
            HighwayLayer.ZoomLevelSet.ZoomLevel12.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level14;
            HighwayLayer.ZoomLevelSet.ZoomLevel15.DefaultLineStyle = LineStyles.Highway3;
            HighwayLayer.ZoomLevelSet.ZoomLevel15.DefaultTextStyle = TextStyles.Highway2("FENAME");
            HighwayLayer.ZoomLevelSet.ZoomLevel15.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            //ShapeFileFeatureLayer SecondaryLayer = new ShapeFileFeatureLayer(Path.Combine(INmapbase, @"Elkhart\IN039lkA.shp"), Path.Combine(INmapbase, @"Elkhart\IN039Secondary.idx"), ShapeFileReadWriteMode.ReadOnly);
            ShapeFileFeatureLayer SecondaryLayer = new ShapeFileFeatureLayer(ShpFilenameRoads, IdxFilenameSecondary, ShapeFileReadWriteMode.ReadOnly);
            SecondaryLayer.ZoomLevelSet.ZoomLevel12.DefaultLineStyle = LineStyles.SecondaryRoad2;
            SecondaryLayer.ZoomLevelSet.ZoomLevel12.DefaultTextStyle = TextStyles.Highway2("FENAME");
            SecondaryLayer.ZoomLevelSet.ZoomLevel12.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level14;
            SecondaryLayer.ZoomLevelSet.ZoomLevel15.DefaultLineStyle = LineStyles.SecondaryRoad2;
            SecondaryLayer.ZoomLevelSet.ZoomLevel15.DefaultTextStyle = TextStyles.SecondaryRoad2("FENAME");
            SecondaryLayer.ZoomLevelSet.ZoomLevel15.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            //// Test after other layers of same file are open - This DOES work.
            //ShapeFileFeatureLayer AllRoads = new ShapeFileFeatureLayer(Path.Combine(INmapbase, @"Elkhart\IN039lkA.shp"));
            //AllRoads.Open();
            //DataTable dtTest = AllRoads.QueryTools.ExecuteQuery("Select FENAME From IN039lkA Where FENAME Like('%County%');");
            //Console.WriteLine("Counter " + TestCounter++ + " " + dtTest.Rows.Count);
            ////foreach (DataRow row in dtTest.Rows)
            ////{
            ////    Console.WriteLine(row["FENAME"]); // Just output to console for testing
            ////}
            //AllRoads.Close();

            //ShapeFileFeatureLayer RoadLayer = new ShapeFileFeatureLayer(Path.Combine(INmapbase, @"Elkhart\IN039lkA.shp"), Path.Combine(INmapbase, @"Elkhart\IN039LocalRoad.idx"), ShapeFileReadWriteMode.ReadOnly);
            ShapeFileFeatureLayer RoadLayer = new ShapeFileFeatureLayer(ShpFilenameRoads, IdxFilenameLocal, ShapeFileReadWriteMode.ReadOnly);
            RoadLayer.Open();
            int count = RoadLayer.QueryTools.GetCount();
            //MessageBox.Show("Count = " + count);
            RoadLayer.Close();
            if (count > 30000)
            {
                RoadLayer.ZoomLevelSet.ZoomLevel15.DefaultLineStyle = LineStyles.LocalRoad4;
                RoadLayer.ZoomLevelSet.ZoomLevel15.DefaultTextStyle = TextStyles.LocalRoad3("FENAME");
                RoadLayer.ZoomLevelSet.ZoomLevel15.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level16;
                RoadLayer.ZoomLevelSet.ZoomLevel17.DefaultLineStyle = LineStyles.LocalRoad3;
                RoadLayer.ZoomLevelSet.ZoomLevel17.DefaultTextStyle = TextStyles.LocalRoad3("FENAME");
                RoadLayer.ZoomLevelSet.ZoomLevel17.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            }
            else if (count < 30000 && count > 20000)
            {
                RoadLayer.ZoomLevelSet.ZoomLevel14.DefaultLineStyle = LineStyles.LocalRoad4;
                RoadLayer.ZoomLevelSet.ZoomLevel14.DefaultTextStyle = TextStyles.LocalRoad3("FENAME");
                RoadLayer.ZoomLevelSet.ZoomLevel14.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level15;
                RoadLayer.ZoomLevelSet.ZoomLevel16.DefaultLineStyle = LineStyles.LocalRoad3;
                RoadLayer.ZoomLevelSet.ZoomLevel16.DefaultTextStyle = TextStyles.LocalRoad3("FENAME");
                RoadLayer.ZoomLevelSet.ZoomLevel16.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            }
            else
            {
                RoadLayer.ZoomLevelSet.ZoomLevel13.DefaultLineStyle = LineStyles.LocalRoad4;
                RoadLayer.ZoomLevelSet.ZoomLevel13.DefaultTextStyle = TextStyles.LocalRoad3("FENAME");
                RoadLayer.ZoomLevelSet.ZoomLevel13.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level14;
                RoadLayer.ZoomLevelSet.ZoomLevel15.DefaultLineStyle = LineStyles.LocalRoad3;
                RoadLayer.ZoomLevelSet.ZoomLevel15.DefaultTextStyle = TextStyles.LocalRoad3("FENAME");
                RoadLayer.ZoomLevelSet.ZoomLevel15.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            }
            
            //ShapeFileFeatureLayer RailRoadLayer = new ShapeFileFeatureLayer(Path.Combine(INmapbase, @"Elkhart\IN039lkB.shp"), ShapeFileReadWriteMode.ReadOnly);
            ShapeFileFeatureLayer RailRoadLayer = new ShapeFileFeatureLayer(ShpFilenameRail, ShapeFileReadWriteMode.ReadOnly);
            RailRoadLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = LineStyles.Railway2;
            RailRoadLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            //ShapeFileFeatureLayer WaterLayer = new ShapeFileFeatureLayer(Path.Combine(INmapbase, @"Elkhart\IN039wat.shp"), ShapeFileReadWriteMode.ReadOnly);
            ShapeFileFeatureLayer WaterLayer = new ShapeFileFeatureLayer(ShpFilenameWater, ShapeFileReadWriteMode.ReadOnly);
            WaterLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.Water1;
            WaterLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle = TextStyles.Water1("LANDNAME");
            WaterLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            InMemoryFeatureLayer IntersectionLayer = new InMemoryFeatureLayer();
            IntersectionLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = LineStyles.Highway5;
            IntersectionLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = PointStyles.CreateSimplePointStyle(PointSymbolType.Circle, GeoColor.StandardColors.Red, 12);
            IntersectionLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;


            LayerOverlay MapLayerOverlay = new LayerOverlay();
            MapLayerOverlay.Layers.Add("COUNTY", CountyLayer);
            MapLayerOverlay.Layers.Add("CITY", CityLayer);
            MapLayerOverlay.Layers.Add("LOCALROAD", RoadLayer);
            MapLayerOverlay.Layers.Add("HIGHWAY", HighwayLayer);
            MapLayerOverlay.Layers.Add("SECONDARY", SecondaryLayer);
            MapLayerOverlay.Layers.Add("RAIL", RailRoadLayer);
            MapLayerOverlay.Layers.Add("WATER", WaterLayer);
            MapLayerOverlay.Layers.Add("INTERSECTION", IntersectionLayer);

            mapMap.Overlays.Add("MapOverlay", MapLayerOverlay);

            // Marker Overlay
            LayerOverlay markerOverlay = new LayerOverlay();
            mapMap.Overlays.Add("MarkerOverlay", markerOverlay);

            // Marker layer
            MarkerLayer markerLayer = new MarkerLayer();
            markerOverlay.Layers.Add("MarkerLayer", markerLayer);

            ScaleLineAdornmentLayer scaleLine = new ScaleLineAdornmentLayer();
            scaleLine.Location = AdornmentLocation.LowerLeft;
            scaleLine.XOffsetInPixel = 25f;
            scaleLine.YOffsetInPixel = -25f;

            if (!mapMap.AdornmentOverlay.Layers.Contains("ScaleLineAdornmentLayer"))
                mapMap.AdornmentOverlay.Layers.Add("ScaleLineAdornmentLayer", scaleLine);
            
            // Attempt to filter geocode by specific ZIPs. It still searched outside the ZIP code.
            RoadLayer.Open();
            CountyZips.Clear();
            string Query = "select distinct ZIPL from " + Path.GetFileNameWithoutExtension(ShpFilenameRoads);
            DataTable dt = RoadLayer.QueryTools.ExecuteQuery(Query);
            foreach (DataRow row in dt.Rows)
            {
                string temp = row["ZIPL"].ToString();
                if (!String.IsNullOrEmpty(temp))
                    CountyZips.Add(temp);
            }
            RoadLayer.Close();

            //CityLayer.Open();
            //string Query = "select distinct NAME from " + Path.GetFileNameWithoutExtension(ShpFilenameCity);
            //DataTable dt = CityLayer.QueryTools.ExecuteQuery(Query);
            //foreach (DataRow row in dt.Rows)
            //{
            //    CountyZips.Add(row["NAME"].ToString());
            //}
            //CityLayer.Close();

            SetMaxView();
        }

        private void SetMaxView()
        {
            SetMaxView(true);
        }

        private void SetMaxView(bool RefreshRequired)
        {
            mapMap.CurrentExtent = GetMaxExtent();
            if (RefreshRequired)
                mapMap.Refresh();
        }

        private RectangleShape GetMaxExtent()
        {
            RectangleShape rect = new RectangleShape();
            LayerOverlay MapLayer = mapMap.Overlays["MapOverlay"] as LayerOverlay;
            ShapeFileFeatureLayer CountyLayer = MapLayer.Layers["COUNTY"] as ShapeFileFeatureLayer;
            CountyLayer.Open();
            rect = CountyLayer.GetBoundingBox();
            CountyLayer.Close();
            return rect;
        }

        private void GeocodeLocation()
        {
            MatchResult matchResult = new MatchResult();
            LayerOverlay MapOverlay = mapMap.Overlays["MapOverlay"] as LayerOverlay;
            if (txtLocation.Text.Contains("&"))
            {
                string[] streets = txtLocation.Text.Split('&');
                if (streets.Length != 2)
                    return;

                string street1 = streets[0].Trim();
                string street2 = streets[1].Trim();
                
                //LayerOverlay MapOverlay = mapMap.Overlays["MapOverlay"] as LayerOverlay;
                ShapeFileFeatureLayer StreetLayer = new ShapeFileFeatureLayer(ShpFilenameRoads, ShapeFileReadWriteMode.ReadOnly);
                //ShapeFileFeatureLayer StreetLayer = new ShapeFileFeatureLayer(Path.Combine(INmapbase, @"Lake\IN089lkA.shp"), ShapeFileReadWriteMode.ReadOnly);

                // --------------------------------------------------------------------------
                // This is where I was attempting to use the QueryTools and get the error. 
                //StreetLayer.Open();
                //string cst1 = String.Format("Select FENAME From {0} Where FENAME Like('%{1}%')", Path.GetFileNameWithoutExtension(ShpFilenameRoads), ConvertStreet(street1));
                //string cst2 = String.Format("Select FENAME From {0} Where FENAME Like('%{1}%')", Path.GetFileNameWithoutExtension(ShpFilenameRoads), ConvertStreet(street2));
                //DataTable dt1 = StreetLayer.QueryTools.ExecuteQuery("Select FENAME From IN039lkA Where FENAME Like('%County%');");//cst1);
                //DataTable dt2 = StreetLayer.QueryTools.ExecuteQuery(cst2);
                //StreetLayer.Close();
                //Collection<Feature> street1features = new Collection<Feature>();
                //Collection<Feature> street2features = new Collection<Feature>();
                //foreach (DataRow row in dt1.Rows)
                //{
                //    Console.WriteLine(row["FENAME"]);
                //    Collection<Feature> temp = StreetLayer.FeatureSource.GetFeaturesByColumnValue("FENAME", (string)row["FENAME"]);
                //    foreach (Feature t in temp)
                //        street1features.Add(t);
                //}
                //foreach (DataRow row in dt2.Rows)
                //{
                //    Console.WriteLine(row["FENAME"]);
                //    Collection<Feature> temp = StreetLayer.FeatureSource.GetFeaturesByColumnValue("FENAME", (string)row["FENAME"]);
                //    foreach (Feature t in temp)
                //        street2features.Add(t);
                //}
                // --------------------------------------------------------------------------

                

                // --------------------------------------------------------------------------
                // This is the working code that I have been using in my testing
                StreetLayer.Open();
                Collection<Feature> street1features = StreetLayer.FeatureSource.GetFeaturesByColumnValue("FENAME", ConvertStreet(street1));//"County Road 17");
                Collection<Feature> street2features = StreetLayer.FeatureSource.GetFeaturesByColumnValue("FENAME", ConvertStreet(street2));//"Suburban");
                StreetLayer.Close();
                // --------------------------------------------------------------------------

                InMemoryFeatureLayer IntersectionLayer = MapOverlay.Layers["INTERSECTION"] as InMemoryFeatureLayer;
                IntersectionLayer.Open();
                IntersectionLayer.EditTools.BeginTransaction();
                Collection<MultilineShape> street1shapes = new Collection<MultilineShape>();
                Collection<MultilineShape> street2shapes = new Collection<MultilineShape>();

                foreach (Feature st1 in street1features)
                {
                    MultilineShape multiLineShape = st1.GetShape() as MultilineShape;
                    street1shapes.Add(multiLineShape);
                    //IntersectionLayer.EditTools.Add(new Feature(multiLineShape));
                }
                foreach (Feature st2 in street2features)
                {
                    MultilineShape multiLineShape = st2.GetShape() as MultilineShape;
                    street2shapes.Add(multiLineShape);
                    //IntersectionLayer.EditTools.Add(new Feature(multiLineShape));
                }

                IntersectionLayer.InternalFeatures.Clear();
                bool IntersectionFound = false;
                MultipointShape intersectionPoints = new MultipointShape();
                try
                {
                    intersectionPoints = GetIntersectionPoints(street1shapes, street2shapes, 50, DistanceUnit.Meter, GeographyUnit.DecimalDegree);
                    IntersectionLayer.EditTools.Add(new Feature(intersectionPoints));
                    IntersectionFound = true;
                }
                catch (Exception ex)
                {
                    MessageBox.Show("No intersection found");
                    IntersectionFound = false;
                }
                
                IntersectionLayer.EditTools.CommitTransaction();

                // --------------------------------------------------------------------------
                // This is where I was attempting to 'zoom in' on the intersection. 
                // The problem is that it doesn't work as expected, i.e. I have a single dot but it won't center and zoom.
                // When I have 2 intersections, as in "Jackson & Main" in Elkhart County, Indiana, it will change the extent 
                // based on both of them as I would want

                // This didn't work for what I was looking for at all.
                //mapMap.CurrentExtent = ExtentHelper.GetBoundingBoxOfItems(IntersectionLayer.InternalFeatures);
                if (IntersectionFound)
                {
                    LayerOverlay markerOverlay = mapMap.Overlays["MarkerOverlay"] as LayerOverlay;
                    MarkerLayer markerLayer = markerOverlay.Layers["MarkerLayer"] as MarkerLayer;
                    markerLayer.MarkerLocation = new PointShape();
                    SetMaxView(false);
                    if (intersectionPoints.Points.Count > 4)
                        mapMap.CenterAt(intersectionPoints.GetCenterPoint());
                    else
                        mapMap.ZoomIntoCenter(92, intersectionPoints.GetCenterPoint());
                }
                else
                    SetMaxView(false);
                    
                IntersectionLayer.Close();

                mapMap.Refresh();
                return;
            }

            // This is code I was using to geocode a standard address entered. It works fine and is pretty quick
            // provided data is formatted properly. I did have problems however limiting the matching to certain Zip codes only.
            
            Collection<MatchResult> matches = new Collection<MatchResult>();

            try
            {
                VerifyGeocoder.Open();
                //matchResult = VerifyGeocoder.Match(txtLocation.Text);

                MatchResult temp = VerifyGeocoder.Match(txtLocation.Text);
                if (temp != null && temp.MatchItems.Count > 0)
                    matches.Add(temp);

                //foreach (string zip in CountyZips)
                //{
                //    MatchResult temp = VerifyGeocoder.Match(txtLocation.Text, zip);
                //    if (temp != null && temp.MatchItems.Count > 0)
                //        matches.Add(temp);
                //}
            }
            finally
            {
                VerifyGeocoder.Close();
            }

            //if (matchResult.MatchItems.Count < 1)
            //{
            //    MessageBox.Show("Address not found, please select a location on the map.");
            //    return;
            //}

            if (matches.Count < 1)
            {
                MessageBox.Show("Address not found, please select a location on the map.");
                return;
            }

            //bool CountyMatchFound = false;
            //MatchItem matchItem = new MatchItem();
            MatchItem matchItem = null;
            
            //------------------------------------------
            // Generating multiple matches by all locations in list
            //Collection<MatchResult> MatchesForSelection = new Collection<MatchResult>();
            //bool MultipleMatches = false;
            //foreach (MatchResult match in matches)
            //{
            //    if (match.MatchItems.Count > 1)
            //    {
            //        MultipleMatches = true;
            //        MatchesForSelection.Add(match);
            //    }
            //    if (match.MatchItems.Count == 1)
            //    {
            //        matchItem = match.MatchItems[0]; // always will have at least one if we have any results at all
            //    }
            //}
            //-----------------------------------------


            Collection<MatchResult> MatchesForSelection = new Collection<MatchResult>();
            Collection<MatchItem> MatchItemsForSelection = new Collection<MatchItem>();
            bool MultipleMatches = false;
            foreach (MatchResult match in matches)
            {
                if (match.MatchItems.Count > 1)
                {
                    MultipleMatches = true;
                    MatchesForSelection.Add(match);
                    foreach (MatchItem item in match.MatchItems.Where(c => c.MatchPairs["State"] == "IN" && c.MatchPairs["County"] == CountyName.ToUpper()))
                    {
                        if (item.MatchPairs.Keys.Contains("Zip") && item.MatchPairs.Keys.Contains("County"))
                        {
                            if (CountyZips.Where(c => c == item.MatchPairs["Zip"]).Count() > 0)// && item.MatchPairs["County"] == CountyName.ToUpper())
                            {
                                MatchItemsForSelection.Add(item);
                            }
                        }
                            
                    }
                }
                if (match.MatchItems.Count == 1)
                {
                    matchItem = match.MatchItems[0]; // always will have at least one if we have any results at all
                }
            }


            if (MultipleMatches)
            {
                using (AddressSelectionDialog asd = new AddressSelectionDialog())
                {
                    //asd.Matches = MatchesForSelection;
                    asd.MatchItems = MatchItemsForSelection;
                    if (asd.ShowDialog() != DialogResult.Cancel)
                    {
                        matchItem = asd.SelectedLocation;
                    }
                    else
                        matchItem = null;
                }
            }

            //foreach (MatchResult match in matches)
            //{
            //    if (!CountyMatchFound)
            //        if (match.MatchItems.Count > 0)
            //        {
            //            foreach (MatchItem item in match.MatchItems.Where(c => c.MatchPairs["State"] == "IN" && c.MatchPairs["County"] == CountyName.ToUpper()))//.MatchItems)
            //            {
            //                //Console.WriteLine(item);
            //                if (!CountyMatchFound)
            //                {
            //                    //IEnumerable<string> zips = CountyZips.Where(c => c == item.MatchPairs["Zip"]);
            //                    //if (CountyZips.Contains(item.MatchPairs["Zip"]))
            //                    if (item.MatchPairs.Keys.Contains("Zip") && item.MatchPairs.Keys.Contains("County"))
            //                        if (CountyZips.Where(c => c == item.MatchPairs["Zip"]).Count() > 0)// && item.MatchPairs["County"] == CountyName.ToUpper())
            //                        {
            //                            CountyMatchFound = true;
            //                            matchItem = item;
            //                        }
            //                }
            //            }
            //        }
            //}

            //matchItem = matchResult.MatchItems.Where(m => m.Name == "StreetMatchResult" && CountyZips.Contains(m.MatchPairs["Zip"])).FirstOrDefault();
            //IEnumerable<MatchItem> matchitems = matchResult.MatchItems.Where(m => m.Name == "StreetMatchResult" && CountyZips.Contains(m.MatchPairs["Zip"]));
            //MatchItem matchItem = matchResult.MatchItems[0];
            if (matchItem != null)
            {
                LayerOverlay markerOverlay = mapMap.Overlays["MarkerOverlay"] as LayerOverlay;
                MarkerLayer markerLayer = markerOverlay.Layers["MarkerLayer"] as MarkerLayer;
                markerLayer.MarkerLocation = new PointShape(matchItem.MatchPairs["CentroidPoint"]);

                // Set the extent around the address and refresh the map
                mapMap.CurrentExtent = new RectangleShape(matchItem.MatchPairs["BoundingBox"]);
            }
            else
            {
                MessageBox.Show("Address not found, please select a location on the map.");
                return;
            }
            
            mapMap.Refresh();
        }

        //Gets the intersection point(s) based on the distance tolerance from the end points of the two collection of MultiLineShapes.
        //We use a tolerance because we want to have an intersection being counted as such even if the points don't have the exact X and Y values.
        private MultipointShape GetIntersectionPoints(Collection<MultilineShape> multiLineShapes1, Collection<MultilineShape> multiLineShapes2, double toleranceDistance, DistanceUnit toleranceUnit, GeographyUnit mapUnit)
        {
            MultipointShape intersectionMultiPointShape = new MultipointShape();

            MultipointShape multiPointShape1 = GetEndPoints(multiLineShapes1);
            MultipointShape multiPointShape2 = GetEndPoints(multiLineShapes2);

            foreach (PointShape pointShape1 in multiPointShape1.Points)
            {
                foreach (PointShape pointShape2 in multiPointShape2.Points)
                {
                    if (pointShape1.GetDistanceTo(pointShape2, mapUnit, toleranceUnit) <= toleranceDistance)
                    {
                        intersectionMultiPointShape.Points.Add(pointShape1);
                    }
                }
            }
            return intersectionMultiPointShape;
        }

        private MultipointShape GetEndPoints(Collection<MultilineShape> multilineShapes)
        {
            MultipointShape multipointShape = new MultipointShape();
            foreach (MultilineShape multilineShape in multilineShapes)
            {
                foreach (LineShape lineShape in multilineShape.Lines)
                {
                    multipointShape.Points.Add(new PointShape(lineShape.Vertices[0]));
                    multipointShape.Points.Add(new PointShape(lineShape.Vertices[lineShape.Vertices.Count - 1]));
                }
            }
            return multipointShape;
        }

        /// <summary>
        /// Simple conversions for Primary and Secondary road abbreviations.
        /// </summary>
        /// <param name="Street"></param>
        /// <returns></returns>
        private string ConvertStreet(string Street)
        {
            if (String.IsNullOrEmpty(Street))
                return Street;
            string temp = Street.ToUpper();
            if (temp.Contains("SR"))
                return Regex.Replace(temp, "SR(?<num>\\d[0-9]*)", @"State Road ${num}");
            if (temp.Contains("CR"))
                return Regex.Replace(temp, "CR(?<num>\\d[0-9]*)", @"County Road ${num}");
            if (temp.Contains("US"))
                return Regex.Replace(temp, "US(?<num>\\d[0-9]*)", @"United States Highway ${num}");
            if (Regex.Match(temp, "I\\d[0-9]+").Success == true)
                return Regex.Replace(temp, "I(?<num>\\d[0-9]*)", @"I-${num}");

            return Street;
        }

        #endregion
    }
}
