﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using ThinkGeo.MapSuite.DesktopEdition;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.MapSuiteGeocoder;
using System.IO;
using System.Collections.ObjectModel;

namespace ThinkGeoGIS_PoC.Forms
{
    public partial class ZoomMapWithStreetDetail : Form
    {
        #region Private Instance Fields

        private string mapbase = Path.Combine(Environment.CurrentDirectory, "MapData");

        #endregion

        #region Constructors

        public ZoomMapWithStreetDetail()
        {
            InitializeComponent();

            this.Load += new EventHandler(ZoomMapWithStreetDetail_Load);
            this.btnZoomIn.Click += new EventHandler(btnZoomIn_Click);
            this.btnZoomOut.Click += new EventHandler(btnZoomOut_Click);
            this.btnSearch.Click += new EventHandler(btnSearch_Click);
            this.lstResult.SelectedIndexChanged += new EventHandler(lstResult_SelectedIndexChanged);
        }

        #endregion

        #region Events

        protected void lstResult_SelectedIndexChanged(object sender, EventArgs e)
        {
            MatchItem matchItem = ((ListItem)lstResult.SelectedItem).MatchItem;

            // Set the marker location to the address selected
            LayerOverlay markerOverlay = wfmapMap.Overlays["MarkerOverlay"] as LayerOverlay;
            MarkerLayer markerLayer = markerOverlay.Layers["MarkerLayer"] as MarkerLayer;
            markerLayer.MarkerLocation = new PointShape(matchItem.MatchPairs["CentroidPoint"]);

            // Set the extent around the address and refresh the map
            wfmapMap.CurrentExtent = new RectangleShape(matchItem.MatchPairs["BoundingBox"]);
            wfmapMap.Refresh();
        }

        protected void btnSearch_Click(object sender, EventArgs e)
        {
            //// Get the path to the data files and create the Geocoder
            //string dataPath = Path.Combine(mapbase, "ChicagoData");
            //UsaGeocoder usaGeocoder = new UsaGeocoder(dataPath, MatchStatus.FuzzyMatch);

            //// Open the geocoder, get any matches and close it
            //MatchResult matchResult;
            //try
            //{
            //    usaGeocoder.Open();
            //    matchResult = usaGeocoder.Match(txtAddress.Text);
            //}
            //finally
            //{
            //    usaGeocoder.Close();
            //}

            //PopulateAddressResultList(matchResult);

            LayerOverlay StaticStreetOverlay = wfmapMap.Overlays["StaticStreetOverlay"] as LayerOverlay;
            ShapeFileFeatureLayer streetLayer = StaticStreetOverlay.Layers["StreetLayer"] as ShapeFileFeatureLayer;
            streetLayer.Open();
            //Collection<Feature> features = streetLayer.FeatureSource.GetFeaturesByColumnValue("RECID", "12503");
            //Collection<Feature> features = streetLayer.FeatureSource.GetFeaturesByColumnValue("NAME", "12th  St");
            Collection<Feature> features = streetLayer.FeatureSource.GetAllFeatures(ReturningColumnsType.AllColumns);
            Collection<Feature> streetfeatures = new Collection<Feature>();
            streetLayer.Close();
            

            LayerOverlay intersectionOverlay = wfmapMap.Overlays["intersections"] as LayerOverlay;
            
            // Street 1 ---------------

            List<MultilineShape> streetShapes = new List<MultilineShape>();
            foreach (Feature f in features.Where(r => r.ColumnValues["NAME"].ToUpper().Contains(txtAddress.Text.ToUpper())))
            {
                streetShapes.Add(f.GetShape() as MultilineShape);
                streetfeatures.Add(f);
            }

            InMemoryFeatureLayer streetInMemoryFeatureLayer = intersectionOverlay.Layers["Street"] as InMemoryFeatureLayer;
            streetInMemoryFeatureLayer.Open();
            streetInMemoryFeatureLayer.InternalFeatures.Clear();
            streetInMemoryFeatureLayer.EditTools.BeginTransaction();
            foreach (MultilineShape m in streetShapes)
            {
                streetInMemoryFeatureLayer.EditTools.Add(new Feature(m));
                streetInMemoryFeatureLayer.EditTools.Add(new Feature(GetEndPoints(m)));
            }
            streetInMemoryFeatureLayer.EditTools.CommitTransaction();
            streetInMemoryFeatureLayer.Close();
            // ----------------------

            // Street 2 ---------------
            List<MultilineShape> streetShapes2 = new List<MultilineShape>();
            foreach (Feature f in features.Where(r => r.ColumnValues["NAME"].ToUpper().Contains(txtCrossStreet.Text.ToUpper())))
            {
                streetShapes2.Add(f.GetShape() as MultilineShape);
                streetfeatures.Add(f);
            }

            InMemoryFeatureLayer streetInMemoryFeatureLayer2 = intersectionOverlay.Layers["Street2"] as InMemoryFeatureLayer;
            streetInMemoryFeatureLayer2.Open();
            streetInMemoryFeatureLayer2.InternalFeatures.Clear();
            streetInMemoryFeatureLayer2.EditTools.BeginTransaction();
            foreach (MultilineShape m in streetShapes2)
            {
                streetInMemoryFeatureLayer2.EditTools.Add(new Feature(m));
                streetInMemoryFeatureLayer2.EditTools.Add(new Feature(GetEndPoints(m)));
            }
            streetInMemoryFeatureLayer2.EditTools.CommitTransaction();
            streetInMemoryFeatureLayer2.Close();
            // ----------------------
            

            wfmapMap.CurrentExtent = ExtentHelper.GetBoundingBoxOfItems(streetfeatures);
            wfmapMap.Refresh();
        }

        protected void btnZoomOut_Click(object sender, EventArgs e)
        {
            wfmapMap.ZoomOut(20);
            wfmapMap.Refresh();
        }

        protected void btnZoomIn_Click(object sender, EventArgs e)
        {
            wfmapMap.ZoomIn(60);
            wfmapMap.Refresh();
        }

        protected void ZoomMapWithStreetDetail_Load(object sender, EventArgs e)
        {
            lblX.Text = lblY.Text = string.Empty;

            wfmapMap.MapUnit = GeographyUnit.DecimalDegree;

            //wfmapMap.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(GeoColor.GeographicColors.ShallowOcean);
            wfmapMap.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(GeoColor.GeographicColors.Grass);

            InMemoryFeatureLayer pointLayer = new InMemoryFeatureLayer();
            pointLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle.PointType = PointType.Bitmap;
            pointLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle.Image = new GeoImage(Path.Combine(mapbase, "Semi Truck.png"));
            pointLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            WorldMapKitWmsDesktopOverlay wmkOverlay = new WorldMapKitWmsDesktopOverlay();
            //wfmapMap.Overlays.Add("WorldMapKitWmsOverlay", wmkOverlay);

            // Austin Streets Layer --------
            ShapeFileFeatureLayer streetLayer = new ShapeFileFeatureLayer(Path.Combine(mapbase, "austinstreets.shp"));
            streetLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = LineStyles.LocalRoad1;
            streetLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            LayerOverlay StaticStreetOverlay = new LayerOverlay();
            StaticStreetOverlay.Layers.Add("StreetLayer", streetLayer);
            wfmapMap.Overlays.Add("StaticStreetOverlay", StaticStreetOverlay);
            // ----------------------------

            // Road lines layer  First Street-----------
            InMemoryFeatureLayer streetInMemoryFeatureLayer = new InMemoryFeatureLayer();
            streetInMemoryFeatureLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = LineStyles.CreateSimpleLineStyle(GeoColor.StandardColors.LightPink, 3, true);
            streetInMemoryFeatureLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = PointStyles.CreateSimplePointStyle(PointSymbolType.Circle, GeoColor.StandardColors.Red, 12);
            streetInMemoryFeatureLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            // ----------------------------

            LayerOverlay pointOverlay = new LayerOverlay();
            pointOverlay.Layers.Add("PointLayer", pointLayer);
            wfmapMap.Overlays.Add("PointOverlay", pointOverlay);
            

            // Road lines layer  Second Street-----------
            InMemoryFeatureLayer streetInMemoryFeatureLayer2 = new InMemoryFeatureLayer();
            streetInMemoryFeatureLayer2.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = LineStyles.CreateSimpleLineStyle(GeoColor.StandardColors.LightBlue, 3, true);
            streetInMemoryFeatureLayer2.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = PointStyles.CreateSimplePointStyle(PointSymbolType.Circle, GeoColor.StandardColors.Blue, 12);
            streetInMemoryFeatureLayer2.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            // ----------------------------

            LayerOverlay intersectionOverlay = new LayerOverlay();
            intersectionOverlay.Layers.Add("Street", streetInMemoryFeatureLayer);
            intersectionOverlay.Layers.Add("Street2", streetInMemoryFeatureLayer2);
            wfmapMap.Overlays.Add("intersections", intersectionOverlay);

            // Marker Overlay
            LayerOverlay markerOverlay = new LayerOverlay();
            wfmapMap.Overlays.Add("MarkerOverlay", markerOverlay);

            // Marker layer
            MarkerLayer markerLayer = new MarkerLayer();
            markerOverlay.Layers.Add("MarkerLayer", markerLayer);

            ScaleLineAdornmentLayer scaleLine = new ScaleLineAdornmentLayer();
            scaleLine.Location = AdornmentLocation.LowerLeft;
            scaleLine.XOffsetInPixel = 25f;
            scaleLine.YOffsetInPixel = -25f;

            wfmapMap.AdornmentOverlay.Layers.Add("ScaleLineAdornmentLayer", scaleLine);

            // ------------------------------------------
            // Test to home in on zip on load
            //bool FilterWorked = false;
            //if (!String.IsNullOrEmpty(CountyFilter))
            //{
            //    // Get the path to the data files and create the Geocoder
            //    string dataPath = Path.Combine(mapbase, "ChicagoData");
            //    UsaGeocoder usaGeocoder = new UsaGeocoder(dataPath, MatchStatus.FuzzyMatch);

            //    // Open the geocoder, get any matches and close it
            //    MatchResult matchResult;
            //    try
            //    {
            //        usaGeocoder.Open();
            //        matchResult = usaGeocoder.Match(String.Format("{0} county IN", CountyFilter));
            //    }
            //    finally
            //    {
            //        usaGeocoder.Close();
            //    }
            //    if (matchResult.MatchItems.Count > 0)
            //    {
            //        FilterWorked = true;
            //        MatchItem matchItem = matchResult.MatchItems[0];
            //        wfmapMap.CurrentExtent = new RectangleShape(matchItem.MatchPairs["BoundingBox"]);
            //    }
            //}
            //if (!FilterWorked)
            //    wfmapMap.CurrentExtent = new RectangleShape(-88.3330001640625, 42.5966329101563, -86.9157638359375, 41.1629170898438);

            // ------------------------------------------

            wfmapMap.CurrentExtent = new RectangleShape(-97.8101, 30.3618, -97.6630, 30.2213);

            //wfmapMap.MapClick += new EventHandler<MapClickWinformsMapEventArgs>(wfmapMap_MapClick);            

            wfmapMap.Refresh();
        }

        protected void wfmapMap_MapClick(object sender, MapClickWinformsMapEventArgs e)
        {
            //MessageBox.Show(string.Format("X:{0}\nY:{1}", e.WorldX.ToString("N4"), e.WorldY.ToString("N4")));
            lblX.Text = string.Format("Lon: {0}", e.WorldX.ToString("N4"));
            lblY.Text = string.Format("Lat: {0}", e.WorldY.ToString("N4"));

            Feature SelectedPoint = new Feature(e.WorldX, e.WorldY, "Point1");

            InMemoryFeatureLayer pointLayer = (InMemoryFeatureLayer)wfmapMap.FindFeatureLayer("PointLayer");
            if (!pointLayer.InternalFeatures.Contains("Point1"))
            {
                pointLayer.InternalFeatures.Add("Point1", SelectedPoint);
            }
            else
            {
                pointLayer.InternalFeatures.Remove("Point1");
                pointLayer.InternalFeatures.Add("Point1", SelectedPoint);
            }

            // Reverse Geocode Location
            //UsaGeocoder usaGeocoder = new UsaGeocoder(Path.Combine(mapbase, "ChicagoData"));
            UsaGeocoder usaGeocoder = new UsaGeocoder(mapbase);

            MatchResult matchResult = new MatchResult();
            try
            {
                usaGeocoder.Open();
                matchResult = usaGeocoder.Match(String.Format("{0} {1}", e.WorldX.ToString(), e.WorldY.ToString()));
            }
            catch (Exception ex)
            {
                MessageBox.Show(String.Format("RevGeocode error:\n{0}", ex.ToString()));
            }
            finally
            {
                usaGeocoder.Close();
            }

            if (matchResult.MatchItems.Count > 0)
            {
                MatchItem matchItem = matchResult.MatchItems[0];
                txtReverseAddress.Text =
                    String.Format("{0} {1} {2} {3}, {4}, {5} County, {6}, {7}",
                    matchItem.MatchPairs["HouseNumber"],
                    matchItem.MatchPairs["PrefixDirectional"],
                    matchItem.MatchPairs["SuffixDirectional"],
                    matchItem.MatchPairs["Street"],
                    matchItem.MatchPairs["StreetType"],
                    matchItem.MatchPairs["City"],
                    matchItem.MatchPairs["County"],
                    matchItem.MatchPairs["State"],
                    matchItem.MatchPairs["Zip"]);
            }
            else
            {
                txtReverseAddress.Text = "Could not find.";
            }

            wfmapMap.Refresh(wfmapMap.Overlays["PointOverlay"]);

        }

        #endregion

        #region Private Instance Methods

        private void PopulateAddressResultList(MatchResult matchResult)
        {
            // Clear the results
            lstResult.Items.Clear();
            //dataGridViewDetail.Rows.Clear();

            // Load the matching items into the grid
            foreach (MatchItem matchItem in matchResult.MatchItems)
            {
                if (matchItem.MatchPairs.ContainsKey("Street"))
                {
                    lstResult.Items.Add(new ListItem(matchItem, new string[] { "Street", "Zip", "State" }));
                }
                else if (matchItem.MatchPairs.ContainsKey("Zip"))
                {
                    lstResult.Items.Add(new ListItem(matchItem, new string[] { "Zip", "State" }));
                }
                else if (matchItem.MatchPairs.ContainsKey("City"))
                {
                    lstResult.Items.Add(new ListItem(matchItem, new string[] { "City", "State" }));
                }
                else if (matchItem.MatchPairs.ContainsKey("County"))
                {
                    lstResult.Items.Add(new ListItem(matchItem, new string[] { "County", "State" }));
                }
                else if (matchItem.MatchPairs.ContainsKey("State"))
                {
                    lstResult.Items.Add(new ListItem(matchItem, new string[] { "State" }));
                }
            }

            // If we find addresses then select the first one to zoom in, if not then say we did not find any
            if (lstResult.Items.Count > 0)
            {
                lstResult.SelectedIndex = 0;
            }
            else
            {
                MessageBox.Show("Address can not be found!", "How Do I Samples", MessageBoxButtons.OK, MessageBoxIcon.Warning, MessageBoxDefaultButton.Button1, MessageBoxOptions.DefaultDesktopOnly);
            }
        }

        private MultipointShape GetEndPoints(MultilineShape multilineShape)
        {
            MultipointShape multipointShape = new MultipointShape();

            foreach (LineShape lineShape in multilineShape.Lines)
            {
                multipointShape.Points.Add(new PointShape(lineShape.Vertices[0]));
                multipointShape.Points.Add(new PointShape(lineShape.Vertices[lineShape.Vertices.Count - 1]));
            }
            return multipointShape;
        }

        #endregion

        #region Public Instance Properties

        public string CountyFilter { get; set; }

        #endregion

    }
}
